#####################################################################
#   Funkcja zwraca konce przedzialu nieodrzucenia H0                #
#   w tescie rozkladu dwumianowego dla pokrycia                     #
#   p - poziom tolerancji VaR                                       #
#   n - dlugosc proby do backtestingu                               #
#   nmax, wmax - rozsadne ograniczenia na odpowiednio               #
#                prawy koniec przedzialu i jego dlugosc             #
#   alpha - poziom istotnosci                                       #
#                                                                   #
#   funkcja zwraca wektor (left, right) z koncami najmniejszego     #
#   przedzialu nieodrzucenia H0 takiego, ze P(<left) >= alpha/2     #
#   i P(>right) >= 1-alpha/2                                       #
#####################################################################

norejInt <- function(p, n, alpha=0.05) {
    pdf <- pbinom(0:n, n, p)

    left <- head((1:n)[pdf - alpha/2 >= 0],1)

    right <- tail((0:n)[pdf -(1- alpha/2) < 0],1) #-1

    int <- c(left, right)
    names(int) <- c("left", "right")
    return(int)
}

########################################################
#   Wykres rolowanego testu metoda swiatel             #
#   actual - zwroty                                    #
#   VaR - prognozy VaR odpowiadajace zwrotom           #
#   thresh - wektor c(ziel, zol) z gornymi granicami   #
#            stref zielonej i zoltej                   #
########################################################

TRplot <- function (actual, VaR, thresh=c(4, 9))
{
    colours <- rollapply(merge(actual, VaR), width=250, function(w){
        ##browser()
        n1 <- sum(w[,2] - w[,1] > 0)
        if(n1 <= thresh[1]) {
            return(0)
        } else if(n1 <= thresh[2]) {
            return(1)
        } else {
            return(2)
        }
    }, align="right", by.column = FALSE)

    plot(index(actual), as.numeric(actual), type = "n", main = NULL,
         ylab = NULL, xlab = NULL, ylim = c(min(actual, VaR), max(actual, VaR)),
         ann = FALSE, cex.main = 0.8, cex.lab = 0.9, cex.axis = 0.8)
    abline(h = 0, col = "grey", lty = 2)
    points(index(actual), as.numeric(actual), pch = 18, col = "lightgrey")
    sel = which(actual < VaR)
    points(index(actual)[sel], as.numeric(actual[sel]), pch = 18, col = "purple")
    lines(index(actual), as.numeric(VaR), lwd = 2, col = "black")


    Vgreen <- Vorange <- Vred <- zoo(NA, order.by=index(VaR))

    idx0 <- index(colours[colours==0])
    Vgreen[idx0] <- VaR[idx0]
    lines(index(Vgreen), Vgreen, lwd = 2, col = "green")

    idx1 <- index(colours[colours==1])
    Vorange[idx1] <- VaR[idx1]
    lines(index(Vorange), Vorange, lwd = 2, col = "orange")


    idx2 <- index(colours[colours==2])
    Vred[idx2] <- VaR[idx2]
    lines(index(Vred), Vred, lwd = 2, col = "red")

    legend("topleft", max(actual), c("returns", "return < VaR", "VaR", "green","yellow","red"),
           col = c("lightgrey", "purple", "black", "green", "orange", "red"), cex = 0.75,
           pch = c(18, 18, -1,-1,-1,-1), lty = c(0, 0, 1,1,1,1), lwd = c(0, 0, 2,2,2,2), bty = "n")
        grid()
    return(invisible())
}


##############################
# Test Kupca                 #
# p - poziom tolerancji VaR  #
# eta - szereg przekroczen   #
##############################

tKupiec <- function(p, eta) {
    eta <- coredata(eta)
    n <- length(eta)
    n
    n1 <- sum(eta)
    n1
    n0 <- n - n1
    n0
    pi <- n1/n
    pi

    #LR_uc   = ( p^n1*(1-p)^n0 ) / ( pi^n1*(1-pi)^n0 )
    LR_uc   = (p/pi)^n1 *((1-p)/(1-pi))^n0
    stat_uc = -2*log(LR_uc)
    prob_uc = 1 - pchisq(q=stat_uc,df=1, lower.tail=TRUE)
    c("n"=n, "n1"=n1, "prob_uc"=prob_uc)
}

#############################################
# Test Christoffersena 1 (niezaleznosc)     #
# p - poziom tolerancji VaR                 #
# eta - szereg przekroczen                  #
#############################################

tChris1 <- function(eta){
    eta1 <- coredata(eta[-length(eta)])
    eta0 <- coredata(eta[-1])
    n00 = sum(!eta1 & !eta0) # no exceedance after no exceedance
    n01 = sum(!eta1 &  eta0)  # exceedance after no exceedance
    n10 = sum( eta1 & !eta0)  # no exceedance after exceedance
    n11 = sum( eta1 &  eta0)   # exceedance after exceedance

    n0  = n00 + n10
    n1  = n01 + n11

    pi01 = n01 / (n00+n01) # prawdopodobienstwo przekroczenia po braku przekroczenia
    pi11 = n11 / (n10+n11) # prawdopodobienstwo przekroczenia po przekroczeniu
    pi   = (n01+n11) / (n00+n01+n10+n11)

    LR_ind   = pi^n1*(1-pi)^n0 / (pi01^n01 * pi11^n11 * (1-pi01)^n00  * (1-pi11)^n10)
# LR_ind   = (pi/pi01)^n01 * (pi/pi11)^n11 * ((1-pi)/(1-pi01))^n00 * ((1-pi)/(1-pi11))^n10
# LR_ind     = ( p^n1*(1-p)^n0 ) / ( pi^n1*(1-pi)^n0 )
    stat_ind = -2*log(LR_ind)
    prob_ind = 1 - pchisq(stat_ind,1)
    prob_ind
}




########################################################
# Test Christoffersena 2 (niezaleznosc i pokrycie)     #
# p - poziom tolerancji VaR                            #
# eta - szereg przekroczen                             #
########################################################

tChris2 <- function(p, eta){
    eta1 <- coredata(eta[-length(eta)])
    eta0 <- coredata(eta[-1])
    n00 = sum(!eta1 & !eta0) # no exceedance after no exceedance
    n01 = sum(!eta1 &  eta0)  # exceedance after no exceedance
    n10 = sum( eta1 & !eta0)  # no exceedance after exceedance
    n11 = sum( eta1 &  eta0)   # exceedance after exceedance

    n0  = n00 + n10
    n1  = n01 + n11

    pi01 = n01 / (n00+n01) # prawdopodobienstwo przekroczenia po braku przekroczenia
    pi11 = n11 / (n10+n11) # prawdopodobienstwo przekroczenia po przekroczeniu
    pi   = (n01+n11) / (n00+n01+n10+n11)
# LR_cc   = p^n0*(1-p)^n1 / (pi01^n01 * pi11^n11 * (1-pi01)^n00  * (1-pi11)^n10)
    LR_cc   = (p/pi01)^n01 * (p/pi11)^n11 * ((1-p)/(1-pi01))^n00 * ((1-p)/(1-pi11))^n10
    stat_cc = -2*log(LR_cc)
    prob_cc = 1 - pchisq(stat_cc,1)
    prob_cc
}

######################################################
#         Wykres zwrotow z VaR i ES                  #
#   (uzupelnienie wykresu VaRplot z rugarch o ES)    #
#   argumenty jak w VaRplot                          #
######################################################

ESVaRplot <- function (alpha, actual, ES, VaR, title = paste("Daily Returns, Expected Shortfall and VaR \nExceedances\n",
    "(alpha=", alpha, ")", sep = ""), ylab = "Daily Log Returns",
    xlab = "Time")
{
    period = diff(index(actual))
    if (attr(period, "units") == "mins") {
        A = as.numeric(actual)
        V = as.numeric(VaR)
        E = as.numeric(ES)
        ep <- axTicksByTime(actual)
        plot(A, type = "n", main = title, ylab = ylab, xlab = xlab,
            ylim = c(min(A, V, E), max(A, V, E)), ann = FALSE, xaxt = "n",
            cex.main = 0.8, cex.lab = 0.9, cex.axis = 0.8)
        axis(1, at = ep, labels = names(ep), tick = TRUE)
        abline(h = 0, col = "grey", lty = 2)
        points(A, pch = 18, col = "lightgrey")
        sel = which(A < V)
        Ap = rep(NA, length(A))
        Ap[sel] = A[sel]
        lines(V, lwd = 1, col = "black")
        lines(E, lwd = 1, col = "blue")
        points(Ap, pch = 3, cex = 1.1, col = "red")
        legend("topleft", max(A), c("returns", "return < VaR",
            "VaR", "ES"), col = c("lightgrey", "red", "black", "blue"), cex = 0.75,
            pch = c(18, 3, -1, -1), lty = c(0, 0, 1, 1), lwd = c(0,
                0, 2, 2), bty = "n")
        grid()
    }
    else {
        plot(index(actual), as.numeric(actual), type = "n", main = title,
            ylab = ylab, xlab = xlab, ylim = c(min(actual, VaR, ES),
                max(actual, VaR, ES)), ann = FALSE, cex.main = 0.8,
            cex.lab = 0.9, cex.axis = 0.8)
        abline(h = 0, col = "grey", lty = 2)
        points(index(actual), as.numeric(actual), pch = 18, col = "lightgrey")
        sel = which(actual < VaR)
        points(index(actual)[sel], as.numeric(actual[sel]), pch = 18,
            col = "red")
        lines(index(actual), as.numeric(VaR), lwd = 2, col = "black")
        lines(index(actual), as.numeric(ES), lwd = 2, col = "blue")
        legend("topleft", max(actual), c("returns", "return < VaR",
            "VaR", "ES"), col = c("lightgrey", "red", "black", "blue"), cex = 0.75,
            pch = c(18, 18, -1, -1), lty = c(0, 0, 1, 1), lwd = c(0,
                0, 2, 2), bty = "n")
        grid()
    }
    return(invisible())
}
